%bcond_with java

%global gtest_url https://github.com/google/googletest
%global gtest_commit 5ec7f0c4a113e2f18ac2c6cc7df51ad6afc24081
%global gtest_dir googletest-%{gtest_commit}
%global so_version 32

Summary:        Protocol Buffers - Google's data interchange format
Name:           protobuf
Version:        3.21.12
Release:        2%{?dist}
License:        BSD
URL:            https://github.com/protocolbuffers/protobuf
Source0:        %{url}/archive/v%{version}/%{name}-%{version}.tar.gz
Source1:        ftdetect-proto.vim
Source2:        protobuf-init.el
Source3:        %{gtest_url}/archive/%{gtest_commit}/%{gtest_dir}.tar.gz

BuildRequires:  make autoconf automake gcc-c++ libtool
BuildRequires:  emacs pkgconfig zlib-devel

%description
Protocol Buffers (a.k.a., protobuf) are Google's language-neutral,
platform-neutral, extensible mechanism for serializing structured
data. 

%package compiler
Summary:        Protocol Buffers compiler
Requires:       %{name} = %{version}-%{release}

%description compiler
This package contains Protocol Buffers compiler for all programming languages.

%package devel
Summary:        Protocol Buffers C++ headers and libraries
Requires:       %{name} = %{version}-%{release}
Requires:       %{name}-compiler = %{version}-%{release}
Requires:       zlib-devel pkgconfig

%description devel
This package contains Protocol Buffers compiler for all languages and
C++ headers and libraries

%package static
Summary:        Static development files for %{name}
Requires:       %{name}-devel = %{version}-%{release}

%description static
Static libraries for Protocol Buffers

%package lite
Summary:        Protocol Buffers LITE_RUNTIME libraries

%description lite
Protocol Buffers built with optimize_for = LITE_RUNTIME.
The "optimize_for = LITE_RUNTIME" option causes the compiler to generate
code which only depends libprotobuf-lite, which is much smaller than
libprotobuf but lacks descriptors, reflection, and some other features.

%package lite-devel
Summary:        Protocol Buffers LITE_RUNTIME development libraries
Requires:       %{name}-devel = %{version}-%{release}
Requires:       %{name}-lite = %{version}-%{release}

%description lite-devel
This package contains development libraries built with
optimize_for = LITE_RUNTIME.
The "optimize_for = LITE_RUNTIME" option causes the compiler 
to generate code which only depends libprotobuf-lite, which
is much smaller than libprotobuf but lacks descriptors,
reflection, and some other features.

%package lite-static
Summary:        Static development files for %{name}-lite
Requires:       %{name}-devel = %{version}-%{release}

%description lite-static
This package contains static development libraries built with
optimize_for = LITE_RUNTIME.
The "optimize_for = LITE_RUNTIME" option causes the compiler to
generate code which only depends libprotobuf-lite, which is much
smaller than libprotobuf but lacks descriptors, reflection, and
some other features.

%package vim
Summary:        Vim syntax highlighting for Google Protocol Buffers descriptions
Requires:       vim-filesystem
BuildArch:      noarch

%description vim
This package contains syntax highlighting for Google Protocol Buffers
descriptions in Vim editor

%package -n python3-%{name}
Summary:        Python bindings for Google Protocol Buffers
BuildRequires:  python3-devel
BuildRequires:  python3dist(setuptools)
BuildRequires:  python3dist(wheel)
Conflicts:      %{name}-compiler > %{version}
Conflicts:      %{name}-compiler < %{version}
Provides:       %{name}-python3 = %{version}-%{release}
BuildArch:      noarch

%description -n python3-%{name}
This package contains Python libraries for Google Protocol Buffers

%if %{with java}
%package java
Summary:        Java Protocol Buffers runtime library
BuildRequires:  maven-local
BuildRequires:  mvn(com.google.code.gson:gson)
BuildRequires:  mvn(com.google.guava:guava)
BuildRequires:  mvn(com.google.guava:guava-testlib)
BuildRequires:  mvn(com.google.truth:truth)
BuildRequires:  mvn(junit:junit)
BuildRequires:  mvn(org.apache.felix:maven-bundle-plugin)
BuildRequires:  mvn(org.apache.maven.plugins:maven-antrun-plugin)
BuildRequires:  mvn(org.apache.maven.plugins:maven-source-plugin)
BuildRequires:  mvn(org.codehaus.mojo:build-helper-maven-plugin)
BuildRequires:  mvn(org.easymock:easymock)
BuildRequires:  mvn(org.mockito:mockito-core)
Conflicts:      %{name}-compiler > %{version}
Conflicts:      %{name}-compiler < %{version}
BuildArch:      noarch

%description java
This package contains Java Protocol Buffers runtime library.

%package javalite
Summary:        Java Protocol Buffers lite runtime library
BuildArch:      noarch

%description javalite
This package contains Java Protocol Buffers lite runtime library.

%package java-util
Summary:        Utilities for Protocol Buffers
BuildArch:      noarch

%description java-util
Utilities to work with protos. It contains JSON support
as well as utilities to work with proto3 well-known types.

%package javadoc
Summary:        Javadoc for %{name}-java
BuildArch:      noarch

%description javadoc
This package contains the API documentation for %{name}-java.

%package parent
Summary:        Protocol Buffer Parent POM
BuildArch:      noarch

%description parent
Protocol Buffer Parent POM.

%package bom
Summary:        Protocol Buffer BOM POM
BuildArch:      noarch

%description bom
Protocol Buffer BOM POM.

%endif

%package emacs
Summary:        Emacs mode for Google Protocol Buffers descriptions
BuildArch:      noarch
Requires:       emacs-filesystem >= %{_emacs_version}

%description emacs
This package contains syntax highlighting for Google Protocol Buffers
descriptions in the Emacs editor.

%prep
%autosetup -n %{name}-%{version} -a 3 -p1
%setup -q -T -D -b 3 -n %{name}-%{version}
rm -rvf 'third_party/googletest'
sed -i -e 's/4.21.12/%{version}/g' python/google/protobuf/__init__.py

mv ../%{gtest_dir} third_party/googletest
find -name \*.cc -o -name \*.h | xargs chmod -x
chmod 644 examples/*.py
rm -f src/solaris/libstdc++.la

%if %{with java}
%pom_remove_dep com.google.errorprone:error_prone_annotations java/util/pom.xml
%pom_remove_dep com.google.j2objc:j2objc-annotations java/util/pom.xml	
%pom_remove_plugin org.codehaus.mojo:animal-sniffer-maven-plugin java/pom.xml
%pom_remove_plugin org.codehaus.mojo:animal-sniffer-maven-plugin java/util/pom.xml

annotations=$(
    find -name '*.java' |
      xargs grep -h -e '^import com\.google\.errorprone\.annotation' \
                    -e '^import com\.google\.j2objc\.annotations' |
      sort -u | sed 's/.*\.\([^.]*\);/\1/' | paste -sd\|
)
find -name '*.java' | xargs sed -ri \
    "s/^import .*\.($annotations);//;s/@($annotations)"'\>\s*(\((("[^"]*")|([^)]*))\))?//g'

%pom_xpath_inject "pom:configuration/pom:instructions" "<Import-Package>sun.misc;resolution:=optional,*</Import-Package>" java/core
%mvn_file :protobuf-java:jar: %{name}/%{name}-java %{name}

%endif


%build
export PTHREAD_LIBS="-lpthread"
./autogen.sh
%configure
%make_build
%{_emacs_bytecompile} editors/protobuf-mode.el

pushd python
%py3_build
popd

	
%if %{with java}
%pom_disable_module kotlin java/pom.xml
%pom_disable_module kotlin-lite java/pom.xml
%mvn_build -s -- -f java/pom.xml
%endif


%install
%make_install STRIPBINARIES=no INSTALL="%{__install} -p" CPPROG="cp -p"
find %{buildroot} -type f -name "*.la" -exec rm -f {} \;
mkdir -p %{buildroot}%{_emacs_sitelispdir}/%{name}
install -p -m 0644 editors/protobuf-mode.el %{buildroot}%{_emacs_sitelispdir}/%{name}
install -p -m 0644 editors/protobuf-mode.elc %{buildroot}%{_emacs_sitelispdir}/%{name}
mkdir -p %{buildroot}%{_emacs_sitestartdir}
install -p -m 0644 %{SOURCE2} %{buildroot}%{_emacs_sitestartdir}


pushd python
%py3_install
find %{buildroot}%{python3_sitelib} -name \*.py |
  xargs sed -i -e '1{\@^#!@d}'
popd
install -p -m 644 -D %{SOURCE1} %{buildroot}%{_datadir}/vim/vimfiles/ftdetect/proto.vim
install -p -m 644 -D editors/proto.vim %{buildroot}%{_datadir}/vim/vimfiles/syntax/proto.vim

%if %{with java}
%mvn_install
%endif


%check
%make_build check CXXFLAGS="%{build_cxxflags} -Wno-error=type-limits"


%files
%license LICENSE
%doc CHANGES.txt CONTRIBUTORS.txt README.md
%{_libdir}/libprotobuf.so.%{so_version}{,.*}

%files compiler
%license LICENSE
%doc README.md
%{_bindir}/protoc
%{_libdir}/libprotoc.so.%{so_version}{,.*}

%files devel
%dir %{_includedir}/google
%{_includedir}/google/protobuf/
%{_libdir}/libprotobuf.so
%{_libdir}/libprotoc.so
%{_libdir}/pkgconfig/protobuf.pc
%doc examples/add_person.cc examples/addressbook.proto examples/list_people.cc examples/Makefile examples/README.md

%files emacs
%license LICENSE
%{_emacs_sitelispdir}/%{name}/
%{_emacs_sitestartdir}/protobuf-init.el

%files static
%{_libdir}/libprotobuf.a
%{_libdir}/libprotoc.a

%files lite
%license LICENSE
%{_libdir}/libprotobuf-lite.so.%{so_version}{,.*}

%files lite-devel
%{_libdir}/libprotobuf-lite.so
%{_libdir}/pkgconfig/protobuf-lite.pc

%files lite-static
%{_libdir}/libprotobuf-lite.a

%files vim
%license LICENSE
%{_datadir}/vim/vimfiles/ftdetect/proto.vim
%{_datadir}/vim/vimfiles/syntax/proto.vim

%files -n python3-protobuf
%license LICENSE
%dir %{python3_sitelib}/google
%{python3_sitelib}/google/protobuf/
%{python3_sitelib}/protobuf-%{version}-py3.*.egg-info/
%{python3_sitelib}/protobuf-%{version}-py3.*-nspkg.pth
%doc python/README.md
%doc examples/add_person.py examples/list_people.py examples/addressbook.proto

%if %{with java}
%files java -f .mfiles-protobuf-java
%license LICENSE
%doc examples/AddPerson.java examples/ListPeople.java
%doc java/README.md

%files java-util -f .mfiles-protobuf-java-util
%license LICENSE

%files javadoc -f .mfiles-javadoc
%license LICENSE

%files parent -f .mfiles-protobuf-parent
%license LICENSE

%files bom -f .mfiles-protobuf-bom
%license LICENSE

%files javalite -f .mfiles-protobuf-javalite
%license LICENSE
%endif

%changelog
* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 3.21.12-2
- Rebuilt for OpenCloudOS Stream 23.09

* Tue Aug 22 2023 jackeyji <jackeyji@tencent.com> - 3.21.12-1
- upgrade to 3.21.12

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 3.19.6-3
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 3.19.6-2
- Rebuilt for OpenCloudOS Stream 23

* Mon Dec 19 2022 ZoeDong <zoedong@tencent.com> - 3.19.6-1
- Initial build
